from os import path
from functools import partial
from datetime import datetime
from ConfigParser import RawConfigParser, NoSectionError

import py
from swsg.sources import ReSTSource
from swsg.templates import SimpleTemplate
from swsg.projects import Project, remove_project, NonexistingProject

from test_templates import SIMPLE_TEMPLATE_TEXT

SOURCE_CONTENT = (
    u'headline\n'
    u'--------\n'
    u'*important* text'
)


def pytest_funcarg__temp_project(request):
    tmpdir = request.getfuncargvalue('tmpdir')
    projects_filename = str(tmpdir.join('projects.shelve'))
    return Project(str(tmpdir), 'test-project', projects_filename)


def test_make_project_directories(temp_project):
    temp_project.make_project_directories()
    path_join = partial(path.join, temp_project.project_dir)
    assert path.exists(temp_project.path)
    assert path.exists(temp_project.project_dir)
    assert path.exists(path_join('sources'))
    assert path.exists(path_join('templates'))
    assert path.exists(path_join('output'))


def test_sources_property(temp_project):
    temp_project.init()
    list_of_sources = list(temp_project.sources)
    assert list_of_sources == []
    make_source = py.path.local(temp_project.source_dir).ensure
    source1 = make_source('source1.rest')
    source1.check(file=True)
    source1_content = 'content of source1'
    source1.write(source1_content)
    source2 = make_source('source2.rest')
    source2.check(file=True)
    source2_content = 'content of source2'
    source2.write(source2_content)
    list_of_sources = list(temp_project.sources)
    assert len(list_of_sources) == 2
    assert list_of_sources == [
        ReSTSource(source1_content),
        ReSTSource(source2_content)]


def test_templates_property(temp_project):
    temp_project.init()
    list_of_templates = list(temp_project.templates)
    assert list_of_templates == []
    make_template = py.path.local(temp_project.template_dir).ensure
    template1 = make_template('template1.html')
    template1.check(file=True)
    template1_content = 'content of template1'
    template1.write(template1_content)
    template2 = make_template('template2.html')
    template2.check(file=True)
    template2_content = 'content of template2'
    template2.write(template2_content)
    list_of_templates = list(temp_project.templates)
    assert len(list_of_templates) == 2
    assert list_of_templates == [
        (
            SimpleTemplate(template1_content),
            path.join(temp_project.template_dir, str(template1))),
        (
            SimpleTemplate(template2_content),
            path.join(temp_project.template_dir, str(template2)))]


def test_update_projects_file(temp_project):
    temp_project.make_project_directories()
    assert temp_project.created is None
    assert temp_project.last_modified is None
    temp_project.update_projects_file()
    assert temp_project.created is None
    assert isinstance(temp_project.last_modified, datetime)
    temp_project.update_projects_file(True)
    assert isinstance(temp_project.created, datetime)
    assert isinstance(temp_project.last_modified, datetime)


def test_local_config(temp_project):
    has_option = temp_project.config.has_option
    get = temp_project.config.get
    assert isinstance(temp_project.config, RawConfigParser)
    assert not temp_project.config.has_section('general')
    assert not temp_project.config.has_section('genshi')
    assert not temp_project.config.has_section('jinja')
    assert not has_option('general', 'template language')
    assert not has_option('genshi', 'method')
    assert not has_option('genshi', 'doctype')
    assert not has_option('jinja2', 'block_start_string')
    assert not has_option('jinja2', 'block_end_string')
    assert not has_option('jinja2', 'variable_start_string')
    assert not has_option('jinja2', 'variable_end_string')
    assert not has_option('jinja2', 'comment_start_string')
    assert not has_option('jinja2', 'comment_end_string')
    assert not has_option('jinja2', 'trim_blocks')
    py.test.raises(
        NoSectionError,
        "get('general', 'template language') == 'simple'")
    temp_project.init()
    assert temp_project.config.has_section('general')
    assert temp_project.config.has_section('genshi')
    assert has_option('general', 'template language')
    assert has_option('genshi', 'method')
    assert has_option('genshi', 'doctype')
    assert has_option('jinja', 'block_start_string')
    assert has_option('jinja', 'block_end_string')
    assert has_option('jinja', 'variable_start_string')
    assert has_option('jinja', 'variable_end_string')
    assert has_option('jinja', 'comment_start_string')
    assert has_option('jinja', 'comment_end_string')
    assert has_option('jinja', 'trim_blocks')
    assert get('general', 'template language') == 'simple'
    assert get('genshi', 'method') == 'html'
    assert get('genshi', 'doctype') == 'html5'


def test_update_config(temp_project):
    temp_project.init()
    section = 'general'
    temp_project.update_config(section, [('template language', 'jinja2')])
    assert temp_project.config.get(section, 'template language') == 'jinja2'


def test_render_project(temp_project):
    temp_project.init()
    source_path = py.path.local(
        temp_project.source_dir).ensure('temp-source.rest')
    source_path.check(file=True)
    source_path.write(SOURCE_CONTENT)
    template_path = py.path.local(
        temp_project.template_dir).ensure('template.html')
    template_path.check(file=True)
    template_path.write(SIMPLE_TEMPLATE_TEXT)
    list_of_templates = list(temp_project.templates)
    assert len(list(temp_project.sources)) == 1
    assert len(list_of_templates) == 1

    template, template_path = list_of_templates[0]
    return_values = temp_project.render()
    output_path, output = return_values.next()
    source_name, rendered_template = template.render(
        temp_project.source_dir).next()
    assert source_name == 'temp-source.rest'
    assert output == rendered_template
    assert output_path == path.join(
        temp_project.output_dir, 'temp-source.html')
    # there is only one template which assigns to only one source, so there
    # shouldn't be anything left in the generator
    py.test.raises(StopIteration, 'return_values.next()')


def test_save_source(temp_project):
    assert not temp_project.updated_projects_file
    temp_project.init()
    # set ``updated_projects_file`` to False to check whether it is set to True
    # after calling ``save_source``
    temp_project.updated_projects_file = False
    source_filename = 'temp-source'
    absolute_source_filename = path.join(
        temp_project.source_dir, source_filename)
    source = ReSTSource(SOURCE_CONTENT)
    temp_project.save_source(source, source_filename)
    assert temp_project.updated_projects_file
    with open(absolute_source_filename) as fp:
        assert SOURCE_CONTENT == fp.read()


def test_save_template(temp_project):
    assert not temp_project.updated_projects_file
    temp_project.init()
    # set ``updated_projects_file`` to False to check whether it is set to True
    # after calling ``save_source``
    temp_project.updated_projects_file = False
    template_content = 'the template text'
    template_filename = 'template.html'
    absolute_template_filename = path.join(
        temp_project.template_dir, template_filename)
    template = SimpleTemplate(template_content)
    temp_project.save_template(template, template_filename)
    assert temp_project.updated_projects_file
    with open(absolute_template_filename) as fp:
        assert template_content == fp.read()


def test_project_exists(temp_project):
    assert not temp_project.exists
    temp_project.init()
    assert temp_project.exists


def test_remove_project(temp_project):
    assert not temp_project.exists
    py.test.raises(
        NonexistingProject,
        'remove_project(temp_project)')
    temp_project.init()
    assert temp_project.exists
    remove_project(temp_project)
    assert not temp_project.exists



import string
from os import path

from swsg.sources import (ensure_markup_is_valid_and_installed,
    get_source_class_by_markup)


SUPPORTED_TEMPLATE_ENGINES = frozenset(['simple', 'mako', 'jinja2', 'genshi'])


class NonexistingSource(Exception):
    def __init__(self, source_path):
        self.source_path = source_path

    def __str__(self):
        return 'the source {0} does not exist'.format(self.source_path)

    def __repr__(self):
        return '{0}({1})'.format(self.__class__.__name__, self.source_path)


class UnsupportedTemplate(Exception):
    def __init__(self, template_language):
        self.template_language = template_language

    def __str__(self):
        return (
            'the template language {0} does either '
            'not exist or is not supported.').format(self.template_language)

    def __repr__(self):
        return '{0}({1})'.format(
            self.__class__.__name__, self.template_language)


class BaseTemplate(object):
    def __init__(self, text):
        '''
        Abstract base class for implementing template classes.
        '''
        self.text = text
        first_line, sep, rest = text.partition('\n')
        # the directive "sources" is optional. Therefore, the first line is
        # checked whether it starts whith the string "sources:". If it does,
        # the string after "sources:" is first stripped by whitespace and then
        # splitted up by commas to get the file names which belong to the given
        # template
        if first_line.startswith('sources:'):
            self.source_names = map(
                unicode.strip,
                first_line.lstrip('sources:').strip().split(','))
            self.text = rest
        else:
            self.source_names = []
            self.text = first_line + rest

    def get_sources(self, base_path):
        for source_name in self.source_names:
            filename = path.join(base_path, source_name)
            # the markup language is the filename extension without the dot.
            # For example, the content of "foo.rest" will be rendered as ReST
            markup_language = path.splitext(filename)[1].lstrip('.')
            ensure_markup_is_valid_and_installed(markup_language)
            # the filename mentioned in the template does not exist
            # -> raise a custom exception
            try:
                with open(filename) as fp:
                    text = fp.read().decode('utf-8')
            except IOError:
                raise NonexistingSource(filename)
            SourceClass = get_source_class_by_markup(markup_language)
            yield SourceClass(text), source_name

    def get_namespace(self, source):
        rendered_source_text = source.render()
        return {
            'title': source.title,
            'content': rendered_source_text}

    def __eq__(self, other):
        return (
            type(self) == type(other) and
            self.text == other.text and
            self.source_names == other.source_names)

    def __ne__(self, other):
        return not (self == other)

    def __hash__(self):
        return hash(self.text) + hash(tuple(self.source_names))

    def render(self, source):
        raise NotImplementedError


class SimpleTemplate(BaseTemplate):
    'Render templates as described in :pep:`0292`'
    def render(self, source_path):
        template = string.Template(self.text)
        for source, source_name in self.get_sources(source_path):
            rendered_template = template.safe_substitute(
                **self.get_namespace(source))
            yield source_name, rendered_template


class MakoTemplate(BaseTemplate):
    def render(self, source_path):
        # import mako only here because this package is optional
        from mako.template import Template
        template = Template(self.text)
        for source, source_name in self.get_sources(source_path):
            rendered_template = template.render(**self.get_namespace(source))
            yield source_name, rendered_template


class Jinja2Template(BaseTemplate):
    def render(self, source_path, **options):
        # import jinja2 only here because this package is optional
        from jinja2 import Environment
        env = Environment(**options)
        template = env.from_string(self.text)
        for source, source_name in self.get_sources(source_path):
            rendered_template = template.render(**self.get_namespace(source))
            yield source_name, rendered_template


class GenshiTemplate(BaseTemplate):
    def render(self, source_path, **options):
        # import genshi only here because this package is optional
        from genshi.template.markup import MarkupTemplate
        template = MarkupTemplate(self.text)
        for source, source_name in self.get_sources(source_path):
            stream = template.generate(**self.get_namespace(source))
            rendered_template = stream.render(**options)
            yield source_name, rendered_template


def get_template_class_by_template_language(template_language):
    normalized_template_language = template_language.lower()
    templates = [
        (frozenset(('simple',)), SimpleTemplate),
        (frozenset(('mako',)), MakoTemplate),
        (frozenset(('jinja', 'jinja2')), Jinja2Template),
        (frozenset(('genshi',)), GenshiTemplate),
    ]
    for template_identifiers, TemplateClass in templates:
        if normalized_template_language in template_identifiers:
            return TemplateClass
    raise UnsupportedTemplate(normalized_template_language)



import py.test
from swsg.templates import (NonexistingSource, BaseTemplate, SimpleTemplate,
    MakoTemplate, Jinja2Template, GenshiTemplate)
from swsg.sources import ReSTSource, MarkdownSource

SOURCE_NAME = u'temp-source.rest'
SOURCE_TEXT = u'some **important** text'
SIMPLE_TEMPLATE_TEXT = (u'''sources: temp-source.rest
<html><body><h1>{title}</h1><p>{content}</p></body></html>''')

FIRST_REAL_LINE = u'first real line of the template'
TEMPLATE_TEXT_MULTIPLE_SOURCES = (
    u'sources: foo.rest, bar.markdown\n' + FIRST_REAL_LINE)


def test_base_template_init():
    t = BaseTemplate(TEMPLATE_TEXT_MULTIPLE_SOURCES)
    source_names = list(t.source_names)
    assert source_names == [u'foo.rest', u'bar.markdown']
    assert t.text == FIRST_REAL_LINE
    t = BaseTemplate(FIRST_REAL_LINE)
    source_names = list(t.source_names)
    assert source_names == []
    assert t.text == FIRST_REAL_LINE


def test_base_template_get_sources(tmpdir):
    rest_text = u'text in the ReST file'
    foo = tmpdir.ensure('foo.rest')
    foo.check(file=True)
    foo.write(rest_text)
    markdown_text = u'text in the markdown file'
    bar = tmpdir.ensure('bar.markdown')
    bar.check(file=True)
    bar.write(markdown_text)
    t = BaseTemplate(TEMPLATE_TEXT_MULTIPLE_SOURCES)
    sources = t.get_sources(str(tmpdir))
    first_source, first_source_name = sources.next()
    assumed_source = ReSTSource(rest_text)
    assert first_source == assumed_source
    second_source, second_source_name = sources.next()
    assumed_source = MarkdownSource(markdown_text)
    assert second_source == assumed_source


def test_simple_template(tmpdir):
    template_text = SIMPLE_TEMPLATE_TEXT.format(
        title='$title', content='$content')
    template_file = tmpdir.ensure('template.html')
    template_file.check(file=True)
    template_file.write(template_text)

    source_filename = 'temp-source.rest'
    source_filename_path = tmpdir.ensure(source_filename)
    source_filename_path.check(file=True)
    source_filename_path.write(SOURCE_TEXT)

    template = SimpleTemplate(template_text)
    source_names = list(template.source_names)
    assert source_names == [source_filename]
    expected_result = (
        u'<html>'
          u'<body>'
            u'<h1>some **important** text</h1>'
            u'<p>'
              u'<p>some <strong>important</strong> text</p>\n'
            u'</p>'
          u'</body>'
        u'</html>')
    rendered_templates = template.render(str(tmpdir))
    source, output = rendered_templates.next()
    # make sure that there was rendered only one template
    py.test.raises(StopIteration, 'rendered_templates.next()')
    assert output == expected_result


def test_mako_template(tmpdir):
    mako = py.test.importorskip('mako')
    from mako import template
    # almost completely copied from mako's homepage
    template_text = u'''<%
    rows = [[v for v in range(0,10)] for row in range(0,10)]
%>
<table>
    % for row in rows:
        ${makerow(row)}
    % endfor
</table>
   
<%def name="makerow(row)">
    <tr>
    % for name in row:
        <td>${name}</td>
    % endfor
    </tr>
</%def>'''
    mako_template = template.Template(template_text)
    rendered_mako_template = mako_template.render()
    text = u'sources: {0}\n{1}'.format(SOURCE_NAME, template_text)
    swsg_mako_template = MakoTemplate(text)
    assert swsg_mako_template.text == template_text
    assert swsg_mako_template.source_names == [SOURCE_NAME]
    nonworking_template_generator = swsg_mako_template.render(str(tmpdir))
    # the source does not exist yet, so it cannot be rendered
    py.test.raises(NonexistingSource, 'nonworking_template_generator.next()')
    source_filename_path = tmpdir.ensure(SOURCE_NAME)
    source_filename_path.check(file=True)
    source_filename_path.write(SOURCE_TEXT)
    list_of_sources = list(swsg_mako_template.get_sources(str(tmpdir)))
    assert list_of_sources == [(ReSTSource(SOURCE_TEXT), SOURCE_NAME)]
    template_generator = swsg_mako_template.render(str(tmpdir))
    received_source_name, output = template_generator.next()
    assert received_source_name == SOURCE_NAME
    # make sure that there was rendered only one template
    py.test.raises(StopIteration, 'template_generator.next()')
    assert output == rendered_mako_template


def test_jinja2_template(tmpdir):
    jinja2 = py.test.importorskip('jinja2')
    template_text = u'''{% for item in range(10) -%}
        {{ item }}
    {%- endfor %}'''
    jinja_template = jinja2.Template(template_text)
    rendered_jinja_template = jinja_template.render()
    text = u'sources: {0}\n{1}'.format(SOURCE_NAME, template_text)
    swsg_jinja_template = Jinja2Template(text)
    assert swsg_jinja_template.text == template_text
    assert swsg_jinja_template.source_names == [SOURCE_NAME]
    nonworking_template_generator = swsg_jinja_template.render(str(tmpdir))
    # the source does not exist yet, so it cannot be rendered
    py.test.raises(NonexistingSource, 'nonworking_template_generator.next()')
    source_filename_path = tmpdir.ensure(SOURCE_NAME)
    source_filename_path.check(file=True)
    source_filename_path.write(SOURCE_TEXT)
    list_of_sources = list(swsg_jinja_template.get_sources(str(tmpdir)))
    assert list_of_sources == [(ReSTSource(SOURCE_TEXT), SOURCE_NAME)]
    template_generator = swsg_jinja_template.render(str(tmpdir))
    received_source_name, output = template_generator.next()
    assert received_source_name == SOURCE_NAME
    # make sure that there was rendered only one template
    py.test.raises(StopIteration, 'template_generator.next()')
    assert output == rendered_jinja_template


def test_genshi_template(tmpdir):
    genshi = py.test.importorskip('genshi')
    from genshi.template.markup import MarkupTemplate
    # the following snippet is copied from
    # http://genshi.edgewall.org/wiki/ApiDocs/genshi.template.markup
    # and extended by an assignment at the beginning
    template_text = '''<div xmlns:py="http://genshi.edgewall.org/">
    <ul py:with="items=range(10)">
        <li py:for="item in items">${item}</li>
    </ul>
</div>'''
    genshi_template = MarkupTemplate(template_text)
    stream = genshi_template.generate()
    rendered_genshi_template = stream.render()
    text = u'sources: {0}\n{1}'.format(SOURCE_NAME, template_text)
    swsg_genshi_template = GenshiTemplate(text)
    assert swsg_genshi_template.text == template_text
    assert swsg_genshi_template.source_names == [SOURCE_NAME]
    nonworking_template_generator = swsg_genshi_template.render(str(tmpdir))
    # the source does not exist yet, so it cannot be rendered
    py.test.raises(NonexistingSource, 'nonworking_template_generator.next()')
    source_filename_path = tmpdir.ensure(SOURCE_NAME)
    source_filename_path.check(file=True)
    source_filename_path.write(SOURCE_TEXT)
    list_of_sources = list(swsg_genshi_template.get_sources(str(tmpdir)))
    assert list_of_sources == [(ReSTSource(SOURCE_TEXT), SOURCE_NAME)]
    template_generator = swsg_genshi_template.render(str(tmpdir))
    received_source_name, output = template_generator.next()
    assert received_source_name == SOURCE_NAME
    # make sure that there was rendered only one template
    py.test.raises(StopIteration, 'template_generator.next()')
    assert output == rendered_genshi_template



import os
import shutil
import shelve
import contextlib
from datetime import datetime
from ConfigParser import RawConfigParser

from swsg import NoninstalledPackage
from swsg.loggers import swsg_logger as logger
from swsg.file_paths import DEFAULT_PROJECTS_FILE_NAME, GLOBAL_CONFIGFILE
from swsg.templates import (UnsupportedTemplate, NonexistingSource,
    GenshiTemplate, get_template_class_by_template_language)
from swsg.sources import get_source_class_by_markup

DEFAULT_SETTINGS = {
    'general':
        [('template language', 'simple')],
    'genshi':
        [
            # can be either 'htlm' or 'xhtml'. every other doesn't make
            # any sense for swsg
            ('method', 'html'),
            # can be one of the following values listed in this link:
            # http://genshi.edgewall.org/browser/trunk/genshi/output.py#L82
            ('doctype', 'html5')],
    # see http://jinja.pocoo.org/api/#jinja2.Environment
    'jinja':
        [
            ('block_start_string', '{%'),
            ('block_end_string', '%}'),
            ('variable_start_string', '{{'),
            ('variable_end_string', '}}'),
            ('comment_start_string', '{#'),
            ('comment_end_string', '#}'),
            ('trim_blocks', 'false')]}


class NonexistingProject(Exception):
    pass


class Project(object):

    def __init__(self, path, name,
                 projects_file_name=DEFAULT_PROJECTS_FILE_NAME):
        self.path = os.path.abspath(path)
        self.name = name
        # interpolation is not needed, it is only annoying with obscure errors
        # -> take RawConfigParser instead of SafeConfigParser
        self.config = RawConfigParser()

        # there is no empty value for datetime.datetime, so ``None`` is used
        self.created = None
        self.last_modified = None

        # the following values can currently only be changed by subclassing
        # this class
        self.project_dir = os.path.join(self.path, self.name)
        self.source_dir = os.path.join(self.project_dir, 'sources')
        self.template_dir = os.path.join(self.project_dir, 'templates')
        self.output_dir = os.path.join(self.project_dir, 'output')
        self.config_filename = os.path.join(self.project_dir, 'config.ini')
        self.projects_file_name = projects_file_name

        # True after the projects file was updated
        self.updated_projects_file = False

    def __repr__(self):
        return '{0} "{1}"'.format(self.__class__.__name__, self.name)

    def init(self):
        '''create the specific project directory and add a configuration file
        with default values

        '''
        self.make_project_directories()
        self.reset_config()
        self.read_config()
        self.update_projects_file(new_created=True)

    def make_project_directories(self):
        logger.notice('creating project directories')
        for path_name in (self.source_dir, self.template_dir, self.output_dir):
            logger.info('creating the directory {0}'.format(path_name))
            os.makedirs(path_name)

    @property
    def exists(self):
        dir_paths = (
            self.project_dir, self.source_dir,
            self.template_dir, self.output_dir
        )
        file_paths = (self.config_filename, self.projects_file_name)
        paths = dir_paths + file_paths
        return (
            all(os.path.exists(path) for path in paths) and
            all(os.path.isdir(dir_path) for dir_path in dir_paths) and
            all(os.path.isfile(file_path) for file_path in file_paths) and
            self.updated_projects_file
        )

    @property
    def sources(self):
        for source_name in os.listdir(self.source_dir):
            # the markup language is the filename extension without the dot.
            # For example, the content of "foo.rest" will be rendered as ReST
            markup_language = os.path.splitext(source_name)[1].lstrip('.')
            source_path = os.path.join(self.source_dir, source_name)
            with open(source_path) as fp:
                text = fp.read().decode('utf-8')
            SourceClass = get_source_class_by_markup(markup_language)
            yield SourceClass(text)

    @property
    def templates(self):
        self.read_config()
        template_language = self.config.get('general', 'template language')
        try:
            TemplateClass = get_template_class_by_template_language(
                template_language)
        except UnsupportedTemplate, e:
            logger.critical(str(e))
        for template_name in os.listdir(self.template_dir):
            filename = os.path.join(self.template_dir, template_name)
            with open(filename) as fp:
                file_content = fp.read().decode('utf-8')
            yield TemplateClass(file_content), filename

    def update_projects_file(self, new_created=False):
        # create the directories where the projects file will be saved if they
        # does not exist yet
        path, file = os.path.split(self.projects_file_name)
        if not os.path.exists(path):
            logger.notice('creating the directory {0}'.format(path))
            os.makedirs(path)
        now = datetime.now()
        if new_created:
            logger.notice(
                'creating the projects file {0}'.format(
                    self.projects_file_name))
            self.created = now
        self.last_modified = now
        with contextlib.closing(shelve.open(self.projects_file_name)) as p:
            logger.notice(
                'updating the projects file {0}'.format(
                    self.projects_file_name))
            p[self.project_dir] = self
        self.updated_projects_file = True

    def read_config(self):
        logger.notice('reading the configuration file')
        with open(self.config_filename) as fp:
            self.config.readfp(fp)

    def reset_config(self):
        logger.notice('resetting the configuration file')
        # check if thee is a global config in CONFIGDIR
        if os.path.exists(GLOBAL_CONFIGFILE):
            shutil.copyfile(GLOBAL_CONFIGFILE, self.config_filename)
            return None
        for section, config_items in DEFAULT_SETTINGS.iteritems():
            if not self.config.has_section(section):
                logger.info('add the section {0}'.format(section))
                self.config.add_section(section)
            for option, value in config_items:
                logger.info(
                    'section {0}: setting the option {1} '
                    'to the value {2}'.format(section, option, value))
                self.config.set(section, option, value)
        with open(self.config_filename, 'w') as fp:
            self.config.write(fp)

    def update_config(self, section, config_items):
        '''Change the given values of ``config_items`` (a list of tuples) in
        the section ``section`` and write them into the configuration file
        ``self.config_filename``.

        '''
        self.read_config()
        for option, value in config_items:
            logger.info(
                'section {0}: setting the option {1} to the value {2}'.format(
                    section, option, value))
            self.config.set(section, option, value)
        with open(self.config_filename, 'w') as fp:
            self.config.write(fp)
        self.update_projects_file()

    def render(self):
        logger.notice('starting the rendering process')
        self.read_config()
        for template, template_filename in self.templates:
            # pass the config settings of genshi if the template is
            # a GenshiTemplate
            if isinstance(template, GenshiTemplate):
                options = self.config.items('genshi')
                render_templates = lambda: template.render(
                    self.source_dir, **options)
            else:
                render_templates = lambda: template.render(self.source_dir)
            try:
                rendered_templates = render_templates()
            except (NoninstalledPackage, NonexistingSource), e:
                logger.critical(str(e))
            for source_name, output in rendered_templates:
                head, tail = os.path.split(source_name)
                filename = os.path.splitext(tail)[0]
                output_path = os.path.join(
                    self.output_dir, filename) + '.html'
                logger.info('{0} + {1} -> {2}'.format(
                    source_name, template_filename, output_path))
                yield output_path, output
        logger.notice('finishing the rendering process')

    def save_source(self, source, name):
        logger.notice('saving the source {0} in the directory {1}'.format(
            name, self.source_dir))
        filename = os.path.join(self.source_dir, name)
        with open(filename, 'w') as fp:
            fp.write(source.text)
        self.update_projects_file()

    def save_template(self, template, name):
        logger.notice('saving the template {0} in the directory {1}'.format(
            name, self.template_dir))
        filename = os.path.join(self.template_dir, name)
        with open(filename, 'w') as fp:
            fp.write(template.text)
        self.update_projects_file()


def list_project_instances(projects_file_name=DEFAULT_PROJECTS_FILE_NAME):
    'get all ``Project`` instances which can be found in the projects file'
    with contextlib.closing(shelve.open(projects_file_name)) as projects:
        return projects.values()


def remove_project(project):
    '''remove both the project's directory and its entry in the projects file

    '''
    proj_dir = project.project_dir
    if project.exists:
        shutil.rmtree(proj_dir)
        with contextlib.closing(shelve.open(project.projects_file_name)) as p:
            p.pop(proj_dir)
    else:
        # project does not exist, therefore it cannot be removed
        raise NonexistingProject(
            'The project {0} with its belonging '
            'directory {1} does not exist.'.format(project, proj_dir))



import py
from swsg.cli import parse_args, validate_change_config
from swsg import __version__ as swsg_version
from swsg.sources import SUPPORTED_MARKUP_LANGUAGES
from swsg.templates import SUPPORTED_TEMPLATE_ENGINES

args_without_any_options = parse_args(['list-projects'])


def pytest_generate_tests(metafunc):
    if 'markup' in metafunc.funcargnames:
        for m in SUPPORTED_MARKUP_LANGUAGES:
            metafunc.addcall(funcargs=dict(markup=m))
    elif 'template' in metafunc.funcargnames:
        for t in SUPPORTED_TEMPLATE_ENGINES:
            metafunc.addcall(funcargs=dict(template=t))


def test_verbose():
    assert not args_without_any_options.verbose
    args_with_verbose_short = parse_args(['-v', 'list-projects'])
    assert args_with_verbose_short.verbose
    args_with_verbose_long = parse_args(['--verbose', 'list-projects'])
    assert args_with_verbose_long.verbose


def test_debug():
    assert not args_without_any_options.debug
    args_with_debug_short = parse_args(['-d', 'list-projects'])
    assert args_with_debug_short.debug
    args_with_debug_long = parse_args(['--debug', 'list-projects'])
    assert args_with_debug_long.debug


def test_version(capsys):
    try:
        parse_args(['--version'])
    except SystemExit, e:
        # exit code is 0
        assert e.code == 0
        out, err = capsys.readouterr()
        # there should be no output to STDOUT
        assert out == u''
        # version string is printed to STDERR
        expected_version_string = 'py.test {0}\n'.format(swsg_version)
        assert err == expected_version_string
    except:
        assert False
    else:
        assert False


def test_logfile():
    assert not args_without_any_options.logfile
    logfile_name = 'file.log'
    args_with_logfile_long = parse_args(
        ['--logfile', logfile_name, 'list-projects'])
    assert args_with_logfile_long.logfile == logfile_name
    args_with_logfile_short = parse_args(['-l', logfile_name, 'list-projects'])
    assert args_with_logfile_short.logfile == logfile_name


def test_init():
    # the name of the project is required, so
    # omitting it will finally raise a SystemExit
    py.test.raises(SystemExit, "parse_args(['init'])")
    project_name = 'test project'
    args_with_default_project_directory = parse_args(['init', project_name])
    assert args_with_default_project_directory.name == project_name
    assert args_with_default_project_directory.project_directory == '.'
    proj_dir = '/my/test/dir'
    args_with_explicit_project_directory = parse_args(
        ['init', project_name, '-p', proj_dir])
    assert args_with_explicit_project_directory.project_directory == proj_dir
    args_with_explicit_project_directory = parse_args(
        ['init', project_name, '--project-directory', proj_dir])
    assert args_with_explicit_project_directory.project_directory == proj_dir


def test_change_config(capsys):
    args = parse_args(['change-config'])
    try:
        validate_change_config(args)
    except SystemExit, e:
        # exit code is 2 because it is a CLI error
        assert e.code == 2
        out, err = capsys.readouterr()
        # there should be no output to STDOUT
        assert out == u''
        expected_error_message = (
            'Error: Neither a markup language '
            'nor a template language was given.\n')
        assert err == expected_error_message
    except:
        assert False
    else:
        assert False
    assert args.markup_language is None
    assert args.template_language is None
    py.test.raises(
        SystemExit,
        "parse_args(['change-config', '-t', 'non_existing_template_engine'])")
    py.test.raises(
        SystemExit,
        "parse_args(['change-config', '-m', 'non_existing_markup_language'])")


def test_markup_config(markup):
    args = parse_args(['change-config', '-m', markup])
    assert args.markup_language == markup
    assert args.template_language is None


def test_template_config(template):
    args = parse_args(['change-config', '-t', template])
    assert args.template_language == template
    assert args.markup_language is None


def test_remove_project(tmpdir):
    py.test.raises(SystemExit, "parse_args(['remove-project'])")
    args = parse_args(['remove-project', str(tmpdir)])
    assert args.path == str(tmpdir)



import py
from logbook import FileHandler, INFO, DEBUG, ERROR
from swsg.file_paths import LOGFILE as DEFAULT_LOGFILE
from swsg.cli import get_logging_handler, set_logging_level

from utils import Object


@py.test.mark.skipif('os.name != "nt"')
def test_get_logging_handler_win():
    args = Object()
    args.logfile = None
    py.test.raises(OSError, 'get_logging_handler(args)')


@py.test.mark.skipif('os.name == "nt"')
def test_get_logging_handler():
    args = Object()
    args.logfile = None
    handler = get_logging_handler(args)
    assert isinstance(handler, FileHandler)
    assert handler._filename == DEFAULT_LOGFILE
    assert handler.level == ERROR


def test_set_logging_level():
    logger = Object()
    logger.level_name = None
    args = Object()
    args.verbose = True
    args.debug = False
    new_logger = set_logging_level(args, logger)
    assert new_logger.level_name == INFO
    assert not new_logger.level_name == DEBUG
    args.verbose = False
    args.debug = True
    new_logger = set_logging_level(args, logger)
    assert not new_logger.level_name == INFO
    assert new_logger.level_name == DEBUG



__version__ = '0.2.4'


class NoninstalledPackage(Exception):
    'raised if a required python package is not installed'
    def __init__(self, package_name):
        self.package_name = package_name

    def __str__(self):
        return 'the package {0!r} is not installed'.format(self.package_name)

    def __repr__(self):
        return '{0}({1})'.format(self.__class__.__name__, self.package_name)



'''
About
=====
swsg (static web site generator) is a tool to create static websites using
human-readable markup languages like ReST_. It was highly-inspired by SR_.
The advantage of SWSG against directly using (X)HTML files is that you have one
(or more) templates and do not have to rewrite every your main layout on ebery
single site. You can concentrate on the main content: the text; the content is
seperated from the templates.

Features
========
- supports many markup languages: Markdown_, Creole_, Textile_, ReST_
- supports many template engines: Jinja2_, Mako_, Genshi_, and also a very
  simple template language for those who do not need control structures like
  loops or if-conditions
- supports clevercss_ beside the usual CSS as a markup language for the
  stylesheets -> not yet!
- provides multiple interfaces: until now, only a CLI is implemented, but
  others are planned:

  - PIDA_-Plugin
  - web interface
  - possibly a GTK+ or Qt application, but I think using the PIDA-Plugin is
    more comfortable

Requirements
============
In order to be able to install swsg, you need `Python`_ (version 2.7 or
2.6 with the Python package `argparse`_ installed) and the
following Python packages:

- texttable_
- `py`_
- `progressbar`_
- at least one of the following markup languages: ReST_, markdown_,
  creole_, textile_
- optional: one or more of the following template engines: `jinja2`_, `mako`_, `genshi`_

.. _ReST: http://docutils.sourceforge.net/rst.html
.. _SR: http://bitbucket.org/tiax/sr/overview
.. _markdown: http://daringfireball.net/projects/markdown/
.. _creole: http://www.wikicreole.org/
.. _textile: http://textile.thresholdstate.com/
.. _jinja2: http://jinja.pocoo.org/2/
.. _mako: http://www.makotemplates.org/
.. _Genshi: http://genshi.edgewall.org/
.. _clevercss: http://sandbox.pocoo.org/clevercss/
.. _PIDA: http://pida.co.uk/
.. _argparse: http://code.google.com/p/argparse/
.. _texttable: http://pypi.python.org/pypi/texttable
.. _py: http://pypi.python.org/pypi/py
.. _progressbar: http://code.google.com/p/python-progressbar/
.. _Python: http://python.org
'''

from __future__ import print_function

try:
    from setuptools import setup
except ImportError:
    import distribute_setup
    distribute_setup.use_setuptools()
    from setuptools import setup

import imp

from swsg import __version__

short_description = (
    'SWSG (Static Web Site Generator) is a tool to generate static web pages.')

requirements = ['docutils', 'py', 'texttable', 'progressbar']
try:
    imp.find_module('argparse')
except ImportError:
    requirements.append('argparse')

setup(
    name='swsg',
    description=short_description,
    long_description=__doc__,
    version=__version__,
    author='Simon Liedtke',
    author_email='liedtke.simon@googlemail.com',
    url='http://github.com/derdon/swsg',
    license='WTFPL',
    packages=['swsg'],
    install_requires=requirements,
    extras_require={
        'markdown': ['markdown'],
        'textile': ['textile'],
        'creole': ['creole'],
        'mako': ['mako'],
        'jinja2': ['jinja2']},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        #'Environment :: Console :: Curses',
        #'Environment :: Plugins',
        #'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Text Processing',
        'Topic :: Text Processing :: Markup'],
    entry_points={
        'console_scripts': [
            'swsg-cli = swsg.cli:main',
         ],
    },
)



import py.test
from swsg.sources import (ReSTSource, CreoleSource, TextileSource,
    MarkdownSource)


def test_source_render():
    source = ReSTSource('**text**')
    assert source.render() == u'<p><strong>text</strong></p>\n'


def test_render_rest():
    r'''
>>> ReSTSource(u'simple test').render()
u'<p>simple test</p>\n'
>>> ReSTSource(u'**simple test**').render()
u'<p><strong>simple test</strong></p>\n'
>>> ReSTSource(u'`simple test`').render()
u'<p><cite>simple test</cite></p>\n'
>>> ReSTSource(u'``test``').render()
u'<p><tt class="docutils literal">test</tt></p>\n'
>>> ReSTSource(u'`example`_\n\n.. _example: http://example.com').render()
u'<p><a class="reference external" href="http://example.com">example</a></p>\n'
>>> ReSTSource(u'`example <http://example.com>`_').render()
u'<p><a class="reference external" href="http://example.com">example</a></p>\n'
    '''
    py.test.importorskip('docutils')


def test_render_creole():
    r'''
>>> CreoleSource(u'test').render()
u'<p>test</p>\n'
>>> CreoleSource(u'test\ntest').render()
u'<p>test test</p>\n'
    '''
    py.test.importorskip('creole')


def test_render_textile():
    r'''
>>> TextileSource('_This_ is a *test.*').render()
'<p><em>This</em> is a <strong>test.</strong></p>'
>>> TextileSource('* One\n* Two\n* Three').render()
'<ul>\n\t\t<li>One</li>\n\t\t<li>Two</li>\n\t\t<li>Three</li>\n\t</ul>'
>>> TextileSource('Link to "Slashdot":http://slashdot.org/').render()
'<p>Link to <a href="http://slashdot.org/">Slashdot</a></p>'
    '''
    py.test.importorskip('textile')


def test_render_markdown():
    r'''
>>> MarkdownSource('[Slashdot](http://slashdot.org/ "Slashdot - News for nerds, stuff that matters")').render()
u'<p><a href="http://slashdot.org/" title="Slashdot - News for nerds, stuff that matters">Slashdot</a></p>'
>>> MarkdownSource('---------------------------------------').render()
u'<hr />'
>>> MarkdownSource('![Python](http://python.org/community/logos/python-logo.png "The Python Logo")').render()
u'<p><img alt="Python" src="http://python.org/community/logos/python-logo.png" title="The Python Logo" /></p>'
    '''
    py.test.importorskip('markdown')
